#include <QCoreApplication>

#include <spdlog/sinks/daily_file_sink.h>
#include <spdlog/sinks/syslog_sink.h>
#include <spdlog/sinks/null_sink.h>
#include <spdlog/async.h>
#include <spdlog/async_logger.h>

#include "config.hpp"
#include "spdlog.hpp"

namespace myx {

namespace log {

static std::shared_ptr< spdlog::logger > logger = nullptr;
static std::shared_ptr< spdlog::logger > logger_mt = nullptr;


std::shared_ptr< spdlog::logger > default_logger()
{
	return( logger );
}


std::shared_ptr< spdlog::logger > default_mt_logger()
{
	return( logger_mt );
}


void spdlog_init()
{
	auto nameSt = fmt::format( "{}_{}", PROJECT_NAME, "st" );
	auto sinksSt = std::make_shared< std::vector< spdlog::sink_ptr > >();

	#ifndef RELEASE
	sinksSt->push_back( std::make_shared< spdlog::sinks::ansicolor_stdout_sink_st >() );
	sinksSt->push_back( std::make_shared< spdlog::sinks::daily_file_sink_st >( nameSt, 23, 59 ) );
	#endif
	sinksSt->push_back( std::make_shared< spdlog::sinks::syslog_sink_st >( nameSt ) );
	logger = std::make_shared< spdlog::logger >( nameSt, sinksSt->begin(), sinksSt->end() );
	logger->flush_on( spdlog::level::warn );
	spdlog::register_logger( logger );

	spdlog::init_thread_pool( 128, 1 );
	auto nameMt = fmt::format( "{}_{}", PROJECT_NAME, "mt" );
	auto sinksMt = std::make_shared< std::vector< spdlog::sink_ptr > >();
	#ifndef RELEASE
	sinksMt->push_back( std::make_shared< spdlog::sinks::ansicolor_stdout_sink_mt >() );
	sinksMt->push_back( std::make_shared< spdlog::sinks::daily_file_sink_mt >( nameMt, 23, 59 ) );
	#endif
	sinksMt->push_back( std::make_shared< spdlog::sinks::syslog_sink_mt >( nameMt ) );
	logger_mt = std::make_shared< spdlog::async_logger >( nameMt, sinksMt->begin(), sinksMt->end(), spdlog::thread_pool() );
	logger_mt->flush_on( spdlog::level::warn );
	spdlog::register_logger( logger_mt );

	#ifndef QT_NO_DEBUG
	spdlog::set_level( spdlog::level::debug );
	#endif
} // spdlog_init


void qt_message_handler( QtMsgType type, const QMessageLogContext& context, const QString& qMsg )
{
	if ( !logger )
	{
		return;
	}
	Q_UNUSED( context )

	const char* msg = qMsg.toUtf8().constData();

	switch ( type )
	{
	case QtDebugMsg:
		logger->debug( msg );
		break;
	case QtInfoMsg:
		logger->info( msg );
		break;
	case QtWarningMsg:
		logger->warn( msg );
		break;
	case QtCriticalMsg:
		logger->critical( msg );
		break;
	case QtFatalMsg:
		logger->critical( "FATAL: %s", msg );
		QCoreApplication::exit( 1 );
		break;
	}
} // messageHandler

} // namespace log

} // namespace myx
