#ifndef MYX_LOG_SPDLOG_HPP_
#define MYX_LOG_SPDLOG_HPP_

#include <spdlog/spdlog.h>
#include <QMessageLogContext>
#include <QString>

namespace myx {

namespace log {

void spdlog_init();
std::shared_ptr< spdlog::logger > default_logger();
std::shared_ptr< spdlog::logger > default_mt_logger();
void qt_message_handler( QtMsgType type, const QMessageLogContext& context, const QString& qMsg );


template < class F, class ... Args >
void trace( F&& format, Args&& ... args ) noexcept
{
	default_logger()->trace( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void debug( F&& format, Args&& ... args ) noexcept
{
	default_logger()->debug( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void info( F&& format, Args&& ... args ) noexcept
{
	default_logger()->info( std::forward< F >( format ),
	                        std::forward< Args >( args )... );
}


template < class F, class ... Args >
void warn( F&& format, Args&& ... args ) noexcept
{
	default_logger()->warn( std::forward< F >( format ),
	                        std::forward< Args >( args )... );
}


template < class F, class ... Args >
void error( F&& format, Args&& ... args ) noexcept
{
	default_logger()->error( std::forward< F >( format ),
	                         std::forward< Args >( args )... );
}


template < class F, class ... Args >
void log( spdlog::level::level_enum lv, F&& format, Args&& ... args ) noexcept
{
	default_logger()->log( lv, std::forward< F >( format ),
	                       std::forward< Args >( args )... );
}

} // namespace log

} // namespace myx


#endif // ifndef MYX_LOG_SPDLOG_HPP_
